function [prob_inclusion,ESS,Beta,L,acceptance]=AG_IWLS(Dataset,prior_intrcp,c1,c,N,P,thita,num_iter,num_burn,num_thin,varargin)

%AG_IWLS  This M-File calculates the posterior gene inclusion probabilities and their effective sample size. The Automatic Generic
%         sampler described by Green in the  Chapter "Trans-dimensional Markon chain Monte Carlo "( HSSS (2003)) with Iterated
%         Weighted Least Square approximation is used to explore efficiently the model and parameter space of the Bayesian  variable
%         selection problem in probit regression with many covariates.


%Last Modification: 11 Nov 2008

%%%%%%%%%%%%%%%%
%INPUT variables
%%%%%%%%%%%%%%%%
%Dataset:two matrix
%       n x 1 is the response variable
%       n x p contains the expession levels of all the genes.This matrix
%       has been centered.
%Prior_intrcp : character
%                'Proper' indicates to use a normal prior for the inercept.
%                'Improper' indicates to use the non-informative improper
%                prior for a location parameter with constant density.
%c1 : matrix 1 x 1
%   Prior variance of the constant term
%c : matrix 1 x 1
%   Prior variance of the regression coefficients.
%N : matrix 1 x 1
%  Maximum number of proposed new variables.
%P : matrix 1 x 1
%  The probability parameter of the binomial distribution used in the model proposal.
%thita : matrix 1 x 1
%      The prior mean of the model size.
%num_iter : matrix 1 x 1
%         Number of iteration of the MCMC.
%num_burn : matrix 1 x 1
%         Burn in period of the MCMC.
%num_thin : matrix 1 x 1
%         Thining of the MCMC.
%Optional Input:
%     numGenes : matrix 1 x 1.
%     The number of pre-selected genes to be included in the design matrix.
%
%%%%%%%%%%%%%%%%%
%OUTPUT variables
%%%%%%%%%%%%%%%%%
%prob_inclusion : matrix p x 1
%                 Posterior gene inclusion probabilities.
%ESS : matrix  p x 1
%    Effective sample size of the posterior gene inclusion probabilities.
%acceptance : matrix 1 x 1
%             Between model acceptance rate of the algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


target=Dataset(:,1);
X=Dataset;X(:,1)=[];

[n,p]=size(X);

if (nargin==11 && p~=varargin{1})
    numGenes=varargin{1};
    numClass=max(target)+1;
    method1='Dudoit';
    newX=select_genes(X,target,numClass,numGenes,method1);
    X=newX.GENES;
    [n,p]=size(X);
end

%Rough estimate of the mean of the auxiliary variable using an MCMC algorithm.
mean_Y=estim_auxil([target X],prior_intrcp,c1,c,N,0,thita,10000,1000,5);


%Initial model
thita=thita/p;
count=sparse(p,p);
model=binornd(1,thita,1,p);
l=find(model);
k=length(l);
t=ones(n,1);
data=X(:,l);

%Model Probability
prob=k*log(thita)+(p-k)*log(1-thita);

betmodelac=0;

%Initial values of the auxiliary variable.
Y(find(target==1),1)=2;Y(find(target==0),1)=-2; % Initial value of Y.


%Log-likelihood of the model and number of predictors.
num=(num_iter-num_burn)/num_thin;
result=sparse(num,p);

%Initialize the cell arrays.
Beta=cell(1,num);
L=cell(1,num);

if isequal(prior_intrcp,'Proper')

    datamat=[t data];
    K=datamat'*datamat+diag([1/c1 1/c*ones(1,k)]);%Precision matrix.
    var=inv(K);%Covariance matrix.

else
    %For improper prior.
    datamat=data;
    K=datamat'*datamat+diag(1/c*ones(1,k));%Precision matrix.
    var=inv(K);%Covariance matrix.

end


%Approximate the covariance matrix of the posterior beta and compute its Cholesky factorization.
int_betalap=inv(datamat'*datamat)*datamat'*mean_Y;
Z_cur=datamat*int_betalap+(target-normcdf(datamat*int_betalap))./normpdf(datamat*int_betalap);
W_cur=diag((normpdf(datamat*int_betalap).^2)./(normcdf(datamat*int_betalap).*normcdf(-(datamat*int_betalap))));
sigmalap=inv(datamat'*W_cur*datamat+diag([1/c1 1/c*ones(1,k)]));
betalap=sigmalap*datamat'*W_cur*Z_cur;
lambda=chol(sigmalap);

%Initial value of beta.
beta=betalap+lambda'*randn(k+1,1);

warning off all

for i=1:num_iter


    %Loglikelihood of the model.
    likmodel=loglikelihood(target,data,beta);

    %Between model move using Trans Dimensional sampler.
    %Propose new gamma.

    proposemodel=model;
    w=1+binornd(N,P); %Block size.
    bin_prob=binopdf(w-1,N,P);

    if k<w
        move=2;
    elseif k==w
        move=1+unidrnd(2);
    else
        move=unidrnd(3);
    end


    if move==1
        %Deletion move

        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k,1,w);
            test=length(unique(r));
        end

        proposemodel(l(r))=0;
        newl=find(proposemodel);
        newk=k-w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);
        newdata=X(:,newl);
        newdatamat=[t X(:,newl)];


        %Probability to move from model to proposemodel.
        r12=1/(3*combination(k,w))*bin_prob;

        %Probability to move from proposemodel to model.
        if newk==w

            r21=1/(2*combination(p-(k-w),w))*bin_prob;

        elseif newk<w

            r21=1/(combination(p-(k-w),w))*bin_prob;

        else

            r21=1/(3*combination(p-(k-w),w))*bin_prob;

        end

        new_int_betalap=inv(newdatamat'*newdatamat)*newdatamat'*mean_Y;
        Z_new=newdatamat*new_int_betalap+(target-normcdf(newdatamat*new_int_betalap))./normpdf(newdatamat*new_int_betalap);
        W_new=diag((normpdf(newdatamat*new_int_betalap).^2)./(normcdf(newdatamat*new_int_betalap).*normcdf(-(newdatamat*new_int_betalap))));
        newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
        newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
        [newlambda,cond]=chol(newsigmalap);

        if (cond~=0)

            Z_new=newdatamat*newbetalap+(target-normcdf(newdatamat*newbetalap))./normpdf(newdatamat*newbetalap);
            W_new=diag((normpdf(newdatamat*newbetalap).^2)./(normcdf(newdatamat*newbetalap).*normcdf(-(newdatamat*newbetalap))));
            newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
            newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
            newlambda=chol(newsigmalap);
            
        end

        Q=inv(lambda')*(beta-betalap);
        v=Q(k-w+2:k+1);

        %Propose new beta.
        proposebeta=newbetalap+newlambda'*Q(1:k-w+1);

        liknewmodel=loglikelihood(target,newdata,proposebeta);%Log-likelihood of new model.

        if isequal(prior_intrcp,'Proper')

            accept=-(likmodel-liknewmodel-beta(1)^2/(2*c1)+proposebeta(1)^2/(2*c1)-w/2*log(c)-0.5/c*beta(2:(k+1))'*beta(2:(k+1))+0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+1/2*v'*v+prob-newprob+log(r12)-log(r21)+sum(log(diag(lambda)))- sum(log(diag(newlambda))));

        else

            accept=-(likmodel-liknewmodel-w/2*log(c)-0.5/c*beta(2:(k+1))'*beta(2:(k+1))+0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+1/2*v'*v+prob-newprob+log(r12)-log(r21)+sum(log(diag(lambda)))- sum(log(diag(newlambda))));

        end


    elseif move==2
        %Adittion move

        covariate=find(model==0);
        k1=length(covariate);
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k1,1,w);
            test=length(unique(r));
        end

        proposemodel(covariate(r))=1;

        newl=find(proposemodel);
        newk=k+w;
        newprob=newk*log(thita)+(p-newk)*log(1-thita);
        newdata=X(:,newl);
        newdatamat=[t X(:,newl)];

        if k<w
            %Probability to move from model to proposemodel.
            r12 = 1/(combination(p-k,w))*bin_prob;


        elseif k==w

            %Proobability to move from model to proposemodel.
            r12 = 1/(2*combination(p-k,w))*bin_prob;

        else
            %Probability to move from model to proposemodel.
            r12 = 1/(3*combination(p-k,w))*bin_prob;

        end

        %Probability to move from proposemodel to model.
        r21=1/(3*combination(k+w,w))*bin_prob;


        %Initial value for the optimization algorithm.
        new_int_betalap=inv(newdatamat'*newdatamat)*newdatamat'*mean_Y;
        Z_new=newdatamat*new_int_betalap+(target-normcdf(newdatamat*new_int_betalap))./normpdf(newdatamat*new_int_betalap);
        W_new=diag((normpdf(newdatamat*new_int_betalap).^2)./(normcdf(newdatamat*new_int_betalap).*normcdf(-(newdatamat*new_int_betalap))));
        newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
        newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
        [newlambda,cond]=chol(newsigmalap);

        if (cond~=0)

            Z_new=newdatamat*newbetalap+(target-normcdf(newdatamat*newbetalap))./normpdf(newdatamat*newbetalap);
            W_new=diag((normpdf(newdatamat*newbetalap).^2)./(normcdf(newdatamat*newbetalap).*normcdf(-(newdatamat*newbetalap))));
            newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
            newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
            newlambda=chol(newsigmalap);
            
        end
        
        v=randn(w,1);
        proposebeta=newbetalap+newlambda'*[inv(lambda')*(beta-betalap);v];

        liknewmodel=loglikelihood(target,newdata,proposebeta);%Log-likelihood of new model.

        if isequal(prior_intrcp,'Proper')

            accept=liknewmodel-likmodel-proposebeta(1)^2/(2*c1)+beta(1)^2/(2*c1)-w/2*log(c)-0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+0.5/c*beta(2:(k+1))'*beta(2:(k+1))+1/2*v'*v+newprob-prob+log(r21)-log(r12)+sum(log(diag(newlambda)))-sum(log(diag(lambda)));

        else

            accept=liknewmodel-likmodel-w/2*log(c)-0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+0.5/c*beta(2:(k+1))'*beta(2:(k+1))+1/2*v'*v+newprob-prob+log(r21)-log(r12)+sum(log(diag(newlambda)))-sum(log(diag(lambda)));

        end

    else
        %Swaption move
        test=0;

        while test~=w %Select w dinstinct predictors.
            r=unidrnd(k,1,w);
            test=length(unique(r));
        end

        proposemodel(l(r))=0;

        covariate=find(model==0);
        k1=length(covariate);
        test=0;

        while test~=w %Select w dinstinct predictors.
            rr=unidrnd(k1,1,w);
            test=length(unique(rr));
        end

        proposemodel(covariate(rr))=1;

        newl=find(proposemodel);
        newdata=X(:,newl);
        newdatamat=[t X(:,newl)];
        newprob=prob;
        newk=k;

        new_int_betalap=inv(newdatamat'*newdatamat)*newdatamat'*mean_Y;
        Z_new=newdatamat*new_int_betalap+(target-normcdf(newdatamat*new_int_betalap))./normpdf(newdatamat*new_int_betalap);
        W_new=diag((normpdf(newdatamat*new_int_betalap).^2)./(normcdf(newdatamat*new_int_betalap).*normcdf(-(newdatamat*new_int_betalap))));
        newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
        newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
        [newlambda,cond]=chol(newsigmalap);

        if (cond~=0)

            Z_new=newdatamat*newbetalap+(target-normcdf(newdatamat*newbetalap))./normpdf(newdatamat*newbetalap);
            W_new=diag((normpdf(newdatamat*newbetalap).^2)./(normcdf(newdatamat*newbetalap).*normcdf(-(newdatamat*newbetalap))));
            newsigmalap=inv(newdatamat'*W_new*newdatamat+diag([1/c1 1/c*ones(1,newk)]));
            newbetalap=newsigmalap*newdatamat'*W_new*Z_new;
            newlambda=chol(newsigmalap);
            
        end
        
        proposebeta=newbetalap+newlambda'*(inv(lambda')*(beta-betalap));

        liknewmodel=loglikelihood(target,newdata,proposebeta);%Log-likelihood of new model.

        if isequal(prior_intrcp,'Proper')

            accept=liknewmodel-likmodel-proposebeta(1)^2/(2*c1)+beta(1)^2/(2*c1)-0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+0.5/c*beta(2:(k+1))'*beta(2:(k+1))+sum(log(diag(newlambda)))-sum(log(diag(lambda)));

        else

            accept=liknewmodel-likmodel-0.5/c*proposebeta(2:(newk+1))'*proposebeta(2:(newk+1))+0.5/c*beta(2:(k+1))'*beta(2:(k+1))+sum(log(diag(newlambda)))-sum(log(diag(lambda)));


        end


    end



    accept=exp(accept);
    u=rand;

    if u<=accept


        beta=proposebeta;
        model=proposemodel;
        data=newdata;
        prob=newprob;
        k=newk;
        l=newl;

        betalap=newbetalap;
        lambda=newlambda;

        if isequal(prior_intrcp,'Proper')

            datamat=[t data];
            K=datamat'*datamat+diag([1/c1 1/c*ones(1,k)]);%Precision matrix.
            var=inv(K);%Covariance matrix.

        else
            %For improper prior.
            datamat=data;
            K=datamat'*datamat+diag(1/c*ones(1,k));%Precision matrix.
            var=inv(K);%Covariance matrix.

        end

        betmodelac=betmodelac+1;


    end

    %Impute the auxiliary variable.
    m=[t data]*beta;

    for j=1:n
        if target(j)==1
            Y(j)=test_truncnorm1(m(j),1,0,+inf);
        else
            Y(j)=test_truncnorm1(m(j),1,-inf,0);
        end
    end

    %Within model move.

    if isequal(prior_intrcp,'Proper')

        mu=var*datamat'*Y;

        %beta=mvnrnd(mu,var)';% Initial sample  of regression coefficients.
        beta=mu+chol(var)'*randn(k+1,1);


    else

        mu=var*data'*Y;
        beta=mu+chol(var)'*randn(k,1);
        intrcp=mean(Y)+1/(sqrt(n))*randn;
        beta=[intrcp;beta];

    end


    if i<num_burn,continue,end

    if i==num_burn
        count=count+sparse(prod_var(l,k),[repmat(l,1,k)],ones(1,k*k),p,p);
        result((i-num_burn)/num_thin+1,l)=1;
        Beta{(i-num_burn)/num_thin+1}=beta';
        L{(i-num_burn)/num_thin+1}=l;

    elseif (mod(i,num_thin)==0)
        count=count+sparse(prod_var(l,k),[repmat(l,1,k)],ones(1,k*k),p,p);
        result((i-num_burn)/num_thin+1,l)=1;
        Beta{(i-num_burn)/num_thin+1}=beta';
        L{(i-num_burn)/num_thin+1}=l;
        
    end

end

prob_inclusion=full(diag(count))/num;

visited_pred=find(sum(count));
result=result(:,visited_pred);

[m_pos,M_pos,f_pos]=summary_integautocorr(result,'Positive');
ESS=num./f_pos';

acceptance=betmodelac/num_iter;

